@@ -1,4 +1,4 @@
-/* $Id: softhsm-keyconv.cpp 3071 2010-03-18 13:45:19Z rb $ */
+/* $Id: softhsm-keyconv.cpp 3642 2010-08-04 09:19:11Z rb $ */
 
 /*
  * Copyright (c) 2009 .SE (The Internet Infrastructure Foundation).
@@ -276,7 +276,7 @@ void to_pkcs8(char *in_path, char *out_path, char *file_pin) {
           error = 1;
           break;
         }
-        if(m != FILE_MAJOR_VERSION || n != FILE_MINOR_VERSION) {
+        if(m > FILE_MAJOR_VERSION_MAX || (m == FILE_MAJOR_VERSION_MAX && n > FILE_MINOR_VERSION_MAX)) {
           fprintf(stderr, "Error: Cannot parse this version of file format, v%i.%i.\n", m, n);
           error = 1;
         }
@@ -370,6 +370,9 @@ void to_pkcs8(char *in_path, char *out_path, char *file_pin) {
       case TAG_EXP2:
       case TAG_COEFF:
       case TAG_PUBVAL:
+      case TAG_CREATED:
+      case TAG_PUBLISH:
+      case TAG_ACTIVATE:
       default:
         break;
     }
