@@ -1,4 +1,4 @@
-/* $Id: softhsm.cpp 2805 2010-02-09 10:38:43Z rb $ */
+/* $Id: softhsm.cpp 3911 2010-09-09 11:22:35Z rb $ */
 
 /*
  * Copyright (c) 2008-2009 .SE (The Internet Infrastructure Foundation).
@@ -47,6 +47,20 @@
 #include <unistd.h>
 #include <iostream>
 #include <fstream>
+#include <sched.h>
+#if defined(HAVE_DLOPEN)
+#include <dlfcn.h>
+#endif
+
+// Includes for the crypto library
+#include <botan/auto_rng.h>
+#include <botan/rsa.h>
+#include <botan/pkcs8.h>
+#include <botan/if_algo.h>
+#include <botan/init.h>
+#include <botan/libstate.h>
+#include <botan/numthry.h>
+using namespace Botan;
 
 void usage() {
   printf("Support tool for libsofthsm\n");
@@ -61,8 +75,17 @@ void usage() {
   printf("                    Use with --file-pin, --slot, --label, --id and --pin.\n");
   printf("  --export <path>   Export a key pair to the given path.\n");
   printf("                    The file will be written in PKCS#8-format.\n");
+  printf("                    Cannot be used in combination with --module,\n");
+  printf("                    since the keys are extracted from the SoftHSM database,\n");
+  printf("                    thus not using PKCS#11.\n");
   printf("                    Use with --file-pin (will encrypt file), --slot, --id\n");
   printf("                    and --pin.\n");
+  printf("  --optimize        Clean up leftovers (session objects in the database) from\n");
+  printf("                    applications that haven't closed down properly.\n");
+  printf("                    Cannot be used in combination with --module.\n");
+  printf("                    Use with --slot and --pin.\n");
+  printf("                    WARNING: Make sure that no application is currently\n");
+  printf("                    using SoftHSM and session objects.\n");
   printf("  --file-pin <PIN>  Supply a PIN if the file is encrypted.\n");
   printf("  --force           Override some warnings.\n");
   printf("  -h                Shows this help screen.\n");
@@ -71,6 +94,7 @@ void usage() {
   printf("                    Use with --force if multiple key pairs may share\n");
   printf("                    the same ID.\n");
   printf("  --label <text>    Defines the label of the object or the token.\n");
+  printf("  --module <path>   Use another PKCS#11 library than SoftHSM.\n");
   printf("  --pin <PIN>       The PIN for the normal user.\n");
   printf("  --slot <number>   The slot where the token is located.\n");
   printf("  --so-pin <PIN>    The PIN for the Security Officer (SO).\n");
@@ -97,8 +121,10 @@ enum {
   OPT_INIT_TOKEN,
   OPT_IMPORT,
   OPT_EXPORT,
+  OPT_OPTIMIZE,
   OPT_SLOT,
   OPT_LABEL,
+  OPT_MODULE,
   OPT_ID,
   OPT_SO_PIN,
   OPT_PIN,
@@ -113,8 +139,10 @@ static const struct option long_options[] = {
   { "init-token",      0, NULL, OPT_INIT_TOKEN },
   { "import",          1, NULL, OPT_IMPORT },
   { "export",          1, NULL, OPT_EXPORT },
+  { "optimize",        0, NULL, OPT_OPTIMIZE },
   { "slot",            1, NULL, OPT_SLOT },
   { "label",           1, NULL, OPT_LABEL },
+  { "module",          1, NULL, OPT_MODULE },
   { "id",              1, NULL, OPT_ID },
   { "so-pin",          1, NULL, OPT_SO_PIN },
   { "pin",             1, NULL, OPT_PIN },
@@ -135,6 +163,7 @@ int main(int argc, char *argv[]) {
   char *userPIN = NULL;
   char *filePIN = NULL;
   char *label = NULL;
+  char *module = NULL;
   char *objectID = NULL;
   char *slot = NULL;
   int forceExec = 0;
@@ -143,8 +172,13 @@ int main(int argc, char *argv[]) {
   int doShowSlots = 0;
   int doImport = 0;
   int doExport = 0;
+  int doOptimize = 0;
   int action = 0;
 
+  moduleHandle = NULL;
+  p11 = NULL;
+  bool was_initialized = false;
+
   while ((opt = getopt_long(argc, argv, "hv", long_options, &option_index)) != -1) {
     switch (opt) {
       case OPT_SHOW_SLOTS:
@@ -165,12 +199,19 @@ int main(int argc, char *argv[]) {
         action++;
         outPath = optarg;
         break;
+      case OPT_OPTIMIZE:
+        doOptimize = 1;
+        action++;
+        break;
       case OPT_SLOT:
         slot = optarg;
         break;
       case OPT_LABEL:
         label = optarg;
         break;
+      case OPT_MODULE:
+        module = optarg;
+        break;
       case OPT_ID:
         objectID = optarg;
         break;
@@ -203,14 +244,36 @@ int main(int argc, char *argv[]) {
   // No action given, display the usage.
   if(action == 0) {
     usage();
+    return 0;
   } else {
-    CK_RV rv = C_Initialize(NULL_PTR);
+    CK_C_GetFunctionList pGetFunctionList = loadLibrary(module);
+    if(pGetFunctionList == NULL) {
+      fprintf(stderr, "Error: Could not load the library.\n");
+      exit(1);
+    }
+    (*pGetFunctionList)(&p11);
+
+    CK_RV rv = p11->C_Initialize(NULL_PTR);
     if(rv != CKR_OK) {
       fprintf(stderr, "Error: Could not initialize libsofthsm. Probably missing the configuration file.\n");
       exit(1);
     }
   }
 
+  // The PKCS#11 library might be using Botan
+  // Check if it has already initialized Botan
+  Library_State* state = swap_global_state(0);
+  // now put it back
+  swap_global_state(state);
+
+  if(state) {
+    was_initialized = true;
+  }
+
+  if(was_initialized == false) {
+    LibraryInitializer::initialize("thread_safe=true");
+  }
+
   // We should create the token.
   if(doInitToken) {
     initToken(slot, label, soPIN, userPIN);
@@ -228,16 +291,87 @@ int main(int argc, char *argv[]) {
 
   // Export a key pair to the given path
   if(doExport) {
-    exportKeyPair(outPath, filePIN, slot, userPIN, objectID);
+    if(module) {
+      fprintf(stderr, "Error: Cannot perform export in combination with the module option.\n");
+    } else {
+      exportKeyPair(outPath, filePIN, slot, userPIN, objectID);
+    }
+  }
+
+  // Remove session objects
+  if(doOptimize) {
+    if(module) {
+      fprintf(stderr, "Error: Cannot perform optimization in combination with the module option.\n");
+    } else {
+      optimize(slot, userPIN);
+    }
+  }
+
+  if(was_initialized == false) {
+    LibraryInitializer::deinitialize();
   }
 
   if(action) {
-    C_Finalize(NULL_PTR);
+    p11->C_Finalize(NULL_PTR);
+    if(moduleHandle) {
+#if defined(HAVE_LOADLIBRARY)
+      // no idea
+#elif defined(HAVE_DLOPEN)
+      dlclose(moduleHandle);
+#endif
+    }
   }
 
   return 0;
 }
 
+// Load the PKCS#11 library
+CK_C_GetFunctionList loadLibrary(char *module) {
+  CK_C_GetFunctionList pGetFunctionList = NULL;
+
+#if defined(HAVE_LOADLIBRARY)
+  // Load PKCS #11 library
+  if(module) {
+    HINSTANCE hDLL = LoadLibrary(_T(module));
+  } else {
+    HINSTANCE hDLL = LoadLibrary(_T(DEFAULT_PKCS11_LIB));
+  }
+
+  if(hDLL == NULL) {
+    // Failed to load the PKCS #11 library
+    return NULL;
+  }
+
+  // Retrieve the entry point for C_GetFunctionList
+  pGetFunctionList = (CK_C_GetFunctionList) GetProcAddress(hDLL, _T("C_GetFunctionList"));
+            
+#elif defined(HAVE_DLOPEN)
+  // Load PKCS #11 library
+  void* pDynLib;
+  if(module) {
+    pDynLib = dlopen(module, RTLD_NOW | RTLD_LOCAL);
+  } else {
+    pDynLib = dlopen(DEFAULT_PKCS11_LIB, RTLD_NOW | RTLD_LOCAL);
+  }
+
+  if(pDynLib == NULL) {
+    // Failed to load the PKCS #11 library
+    return NULL;
+  }
+
+  // Retrieve the entry point for C_GetFunctionList
+  pGetFunctionList = (CK_C_GetFunctionList) dlsym(pDynLib, "C_GetFunctionList");
+
+  // Store the handle so we can dlclose it later
+  moduleHandle = pDynLib;
+
+#else
+  return NULL;
+#endif
+
+  return pGetFunctionList;
+}
+
 // Creates a SoftHSM token at the given location.
 
 void initToken(char *slot, char *label, char *soPIN, char *userPIN) {
@@ -308,7 +442,7 @@ void initToken(char *slot, char *label, char *soPIN, char *userPIN) {
   memset(paddedLabel, ' ', sizeof(paddedLabel));
   memcpy(paddedLabel, label, strlen(label));
 
-  CK_RV rv = C_InitToken(slotID, (CK_UTF8CHAR_PTR)so_pin_copy, soLength, paddedLabel);
+  CK_RV rv = p11->C_InitToken(slotID, (CK_UTF8CHAR_PTR)so_pin_copy, soLength, paddedLabel);
 
   switch(rv) {
     case CKR_OK:
@@ -333,19 +467,19 @@ void initToken(char *slot, char *label, char *soPIN, char *userPIN) {
   }
 
   CK_SESSION_HANDLE hSession;
-  rv = C_OpenSession(slotID, CKF_SERIAL_SESSION | CKF_RW_SESSION, NULL_PTR, NULL_PTR, &hSession);
+  rv = p11->C_OpenSession(slotID, CKF_SERIAL_SESSION | CKF_RW_SESSION, NULL_PTR, NULL_PTR, &hSession);
   if(rv != CKR_OK) {
     fprintf(stderr, "Error: Could not open a session with the library.\n");
     return;
   }
 
-  rv = C_Login(hSession, CKU_SO, (CK_UTF8CHAR_PTR)so_pin_copy, soLength);
+  rv = p11->C_Login(hSession, CKU_SO, (CK_UTF8CHAR_PTR)so_pin_copy, soLength);
   if(rv != CKR_OK) {
     fprintf(stderr, "Error: Could not log in on the token.\n");
     return;
   }
 
-  rv = C_InitPIN(hSession, (CK_UTF8CHAR_PTR)user_pin_copy, userLength);
+  rv = p11->C_InitPIN(hSession, (CK_UTF8CHAR_PTR)user_pin_copy, userLength);
   if(rv != CKR_OK) {
     fprintf(stderr, "Error: Could not initialize the user PIN.\n");
     return;
@@ -356,14 +490,14 @@ void initToken(char *slot, char *label, char *soPIN, char *userPIN) {
 
 void showSlots() {
   CK_ULONG ulSlotCount;
-  CK_RV rv = C_GetSlotList(CK_FALSE, NULL_PTR, &ulSlotCount);
+  CK_RV rv = p11->C_GetSlotList(CK_FALSE, NULL_PTR, &ulSlotCount);
   if(rv != CKR_OK) {
     fprintf(stderr, "Error: Could not get the number of slots.\n");
     return;
   }
 
   CK_SLOT_ID_PTR pSlotList = (CK_SLOT_ID_PTR)malloc(ulSlotCount*sizeof(CK_SLOT_ID));
-  rv = C_GetSlotList(CK_FALSE, pSlotList, &ulSlotCount);
+  rv = p11->C_GetSlotList(CK_FALSE, pSlotList, &ulSlotCount);
   if (rv != CKR_OK) {
     fprintf(stderr, "Error: Could not get the slot list.\n");
     return;
@@ -375,7 +509,7 @@ void showSlots() {
     CK_SLOT_INFO slotInfo;
     CK_TOKEN_INFO tokenInfo;
 
-    rv = C_GetSlotInfo(pSlotList[i], &slotInfo);
+    rv = p11->C_GetSlotInfo(pSlotList[i], &slotInfo);
     if(rv != CKR_OK) {  
       fprintf(stderr, "Error: Could not get the slot info.\n");
       free(pSlotList);
@@ -389,7 +523,7 @@ void showSlots() {
     } else {
       printf("yes\n");
 
-      rv = C_GetTokenInfo(pSlotList[i], &tokenInfo);
+      rv = p11->C_GetTokenInfo(pSlotList[i], &tokenInfo);
       if(rv != CKR_OK) {
         fprintf(stderr, "Error: Could not get the token info.\n");
         return;
@@ -449,7 +583,7 @@ void importKeyPair(char *filePath, char *filePIN, char *slot, char *userPIN, cha
 
   CK_SLOT_ID slotID = atoi(slot);
   CK_SESSION_HANDLE hSession;
-  CK_RV rv = C_OpenSession(slotID, CKF_SERIAL_SESSION | CKF_RW_SESSION, NULL_PTR, NULL_PTR, &hSession);
+  CK_RV rv = p11->C_OpenSession(slotID, CKF_SERIAL_SESSION | CKF_RW_SESSION, NULL_PTR, NULL_PTR, &hSession);
   if(rv != CKR_OK) {
     if(rv == CKR_SLOT_ID_INVALID) {
       fprintf(stderr, "Error: The given slot does not exist.\n");
@@ -460,7 +594,7 @@ void importKeyPair(char *filePath, char *filePIN, char *slot, char *userPIN, cha
     return;
   }
 
-  rv = C_Login(hSession, CKU_USER, (CK_UTF8CHAR_PTR)userPIN, strlen(userPIN));
+  rv = p11->C_Login(hSession, CKU_USER, (CK_UTF8CHAR_PTR)userPIN, strlen(userPIN));
   if(rv != CKR_OK) {
     if(rv == CKR_PIN_INCORRECT) {
       fprintf(stderr, "Error: The given user PIN does not match the one in the token.\n");
@@ -515,11 +649,14 @@ void importKeyPair(char *filePath, char *filePIN, char *slot, char *userPIN, cha
     { CKA_MODULUS,          keyMat->bigN, keyMat->sizeN },
     { CKA_PRIVATE_EXPONENT, keyMat->bigD, keyMat->sizeD },
     { CKA_PRIME_1,          keyMat->bigP, keyMat->sizeP },
-    { CKA_PRIME_2,          keyMat->bigQ, keyMat->sizeQ }
+    { CKA_PRIME_2,          keyMat->bigQ, keyMat->sizeQ },
+    { CKA_EXPONENT_1,       keyMat->bigDMP1, keyMat->sizeDMP1 },
+    { CKA_EXPONENT_2,       keyMat->bigDMQ1, keyMat->sizeDMQ1 },
+    { CKA_COEFFICIENT,      keyMat->bigIQMP, keyMat->sizeIQMP }
   };
 
   CK_OBJECT_HANDLE hKey1, hKey2;
-  rv = C_CreateObject(hSession, privTemplate, 16, &hKey1);
+  rv = p11->C_CreateObject(hSession, privTemplate, 19, &hKey1);
   if(rv != CKR_OK) {
     freeKeyMaterial(keyMat);
     free(objID);
@@ -527,18 +664,18 @@ void importKeyPair(char *filePath, char *filePIN, char *slot, char *userPIN, cha
     return;
   }
 
-  rv = C_CreateObject(hSession, pubTemplate, 10, &hKey2);
+  rv = p11->C_CreateObject(hSession, pubTemplate, 10, &hKey2);
 
   freeKeyMaterial(keyMat);
   free(objID);
 
   if(rv != CKR_OK) {
-    C_DestroyObject(hSession, hKey1);
+    p11->C_DestroyObject(hSession, hKey1);
     fprintf(stderr, "Error: Could not save the public key in the token.\n");
     return;
   }
 
-  printf("The key pair has been imported to the token in slot %i.\n", slotID);
+  printf("The key pair has been imported to the token in slot %lu.\n", slotID);
 }
 
 void exportKeyPair(char *filePath, char *filePIN, char *slot, char *userPIN, char *objectID) {
@@ -573,7 +710,7 @@ void exportKeyPair(char *filePath, char *filePIN, char *slot, char *userPIN, cha
 
   CK_SLOT_ID slotID = atoi(slot);
   CK_SESSION_HANDLE hSession;
-  CK_RV rv = C_OpenSession(slotID, CKF_SERIAL_SESSION | CKF_RW_SESSION, NULL_PTR, NULL_PTR, &hSession);
+  CK_RV rv = p11->C_OpenSession(slotID, CKF_SERIAL_SESSION | CKF_RW_SESSION, NULL_PTR, NULL_PTR, &hSession);
   if(rv != CKR_OK) {
     if(rv == CKR_SLOT_ID_INVALID) {
       fprintf(stderr, "Error: The given slot does not exist.\n");
@@ -584,7 +721,7 @@ void exportKeyPair(char *filePath, char *filePIN, char *slot, char *userPIN, cha
     return;
   }
 
-  rv = C_Login(hSession, CKU_USER, (CK_UTF8CHAR_PTR)userPIN, strlen(userPIN));
+  rv = p11->C_Login(hSession, CKU_USER, (CK_UTF8CHAR_PTR)userPIN, strlen(userPIN));
   if(rv != CKR_OK) {
     if(rv == CKR_PIN_INCORRECT) {
       fprintf(stderr, "Error: The given user PIN does not match the one in the token.\n");
@@ -625,6 +762,100 @@ void exportKeyPair(char *filePath, char *filePIN, char *slot, char *userPIN, cha
   delete privKey;
 }
 
+void optimize(char *slot, char *userPIN) {
+  if(slot == NULL) {
+    fprintf(stderr, "Error: A slot number must be supplied. Use --slot <number>\n");
+    return;
+  }
+
+  if(userPIN == NULL) {
+    fprintf(stderr, "Error: An user PIN must be supplied. Use --pin <PIN>\n");
+    return;
+  }
+
+  CK_SLOT_ID slotID = atoi(slot);
+  CK_SESSION_HANDLE hSession;
+  CK_RV rv = p11->C_OpenSession(slotID, CKF_SERIAL_SESSION | CKF_RW_SESSION, NULL_PTR, NULL_PTR, &hSession);
+  if(rv != CKR_OK) {
+    if(rv == CKR_SLOT_ID_INVALID) {
+      fprintf(stderr, "Error: The given slot does not exist.\n");
+    } else {
+      fprintf(stderr, "Error: Could not open a session on the given slot.\n");
+    }
+    return;
+  }
+
+  rv = p11->C_Login(hSession, CKU_USER, (CK_UTF8CHAR_PTR)userPIN, strlen(userPIN));
+  if(rv != CKR_OK) {
+    if(rv == CKR_PIN_INCORRECT) {
+      fprintf(stderr, "Error: The given user PIN does not match the one in the token.\n");
+    } else {
+      fprintf(stderr, "Error: Could not log in on the token.\n");
+    }
+    return;
+  }
+
+  // Get the path to the token database
+  char *dbPath = getDBPath(slotID);
+  if(dbPath == NULL) {
+    return;
+  }
+
+  if(!removeSessionObjs(dbPath)) {
+    printf("Removed all session objects.\n");
+  }
+
+  free(dbPath);
+}
+
+int removeSessionObjs(char *dbPath) {
+  sqlite3 *db = NULL;
+  const char select_str[] = "SELECT objectID FROM Attributes WHERE type = ? AND value = ?;";
+  const char delete_str[] = "DELETE FROM Objects WHERE objectID = ?;";
+  sqlite3_stmt *select_sql = NULL;
+  sqlite3_stmt *delete_sql = NULL;
+  CK_BBOOL ckFalse = CK_FALSE;
+  int retVal = 0;
+
+  if(sqlite3_open(dbPath, &db) != 0) {
+    fprintf(stderr, "ERROR: Could not connect to database.\n");
+  }
+
+  if(sqlite3_prepare_v2(db, select_str, -1, &select_sql, NULL) != 0) {
+    fprintf(stderr, "ERROR: Could not prepare a SQL statement.\n");
+    sqlite3_close(db);
+    return 1;
+  }
+
+  if(sqlite3_prepare_v2(db, delete_str, -1, &delete_sql, NULL) != 0) {
+    fprintf(stderr, "ERROR: Could not prepare a SQL statement.\n");
+    sqlite3_finalize(select_sql);
+    sqlite3_close(db);
+    return 1;
+  }
+
+  sqlite3_bind_int(select_sql, 1, CKA_TOKEN);
+  sqlite3_bind_blob(select_sql, 2, &ckFalse, sizeof(ckFalse), SQLITE_TRANSIENT);
+
+  while((retVal = sqlite3_step(select_sql)) == SQLITE_BUSY || retVal == SQLITE_ROW) {
+    if(retVal == SQLITE_ROW) {
+      sqlite3_bind_int(delete_sql, 1, sqlite3_column_int(select_sql, 0));
+      while(sqlite3_step(delete_sql) == SQLITE_BUSY) {
+        sched_yield();
+      }
+      sqlite3_reset(delete_sql);
+    } else {
+      sched_yield();
+    }
+  }
+
+  sqlite3_finalize(delete_sql);
+  sqlite3_finalize(select_sql);
+  sqlite3_close(db);
+
+  return 0;
+}
+
 // Convert a char array of hexadecimal characters into a binary representation
 
 char* hexStrToBin(char *objectID, int idLength, int *newLen) {
@@ -735,22 +966,34 @@ key_material_t* importKeyMat(char *filePath, char *filePIN) {
   }
 
   IF_Scheme_PrivateKey *ifKeyPriv = dynamic_cast<IF_Scheme_PrivateKey*>(privKey);
+  BigInt d1 = ifKeyPriv->get_d() % (ifKeyPriv->get_p() - 1);
+  BigInt d2 = ifKeyPriv->get_d() % (ifKeyPriv->get_q() - 1);
+  BigInt c = inverse_mod(ifKeyPriv->get_q(), ifKeyPriv->get_p());
   key_material_t *keyMat = (key_material_t *)malloc(sizeof(key_material_t));
   keyMat->sizeE = ifKeyPriv->get_e().bytes();
   keyMat->sizeN = ifKeyPriv->get_n().bytes();
   keyMat->sizeD = ifKeyPriv->get_d().bytes();
   keyMat->sizeP = ifKeyPriv->get_p().bytes();
   keyMat->sizeQ = ifKeyPriv->get_q().bytes();
+  keyMat->sizeDMP1 = d1.bytes();
+  keyMat->sizeDMQ1 = d2.bytes();
+  keyMat->sizeIQMP = c.bytes();
   keyMat->bigE = (CK_VOID_PTR)malloc(keyMat->sizeE);
   keyMat->bigN = (CK_VOID_PTR)malloc(keyMat->sizeN);
   keyMat->bigD = (CK_VOID_PTR)malloc(keyMat->sizeD);
   keyMat->bigP = (CK_VOID_PTR)malloc(keyMat->sizeP);
   keyMat->bigQ = (CK_VOID_PTR)malloc(keyMat->sizeQ);
+  keyMat->bigDMP1 = (CK_VOID_PTR)malloc(keyMat->sizeDMP1);
+  keyMat->bigDMQ1 = (CK_VOID_PTR)malloc(keyMat->sizeDMQ1);
+  keyMat->bigIQMP = (CK_VOID_PTR)malloc(keyMat->sizeIQMP);
   ifKeyPriv->get_e().binary_encode((byte *)keyMat->bigE);
   ifKeyPriv->get_n().binary_encode((byte *)keyMat->bigN);
   ifKeyPriv->get_d().binary_encode((byte *)keyMat->bigD);
   ifKeyPriv->get_p().binary_encode((byte *)keyMat->bigP);
   ifKeyPriv->get_q().binary_encode((byte *)keyMat->bigQ);
+  d1.binary_encode((byte *)keyMat->bigDMP1);
+  d2.binary_encode((byte *)keyMat->bigDMQ1);
+  c.binary_encode((byte *)keyMat->bigIQMP);
   delete privKey;
 
   return keyMat;
@@ -775,6 +1018,16 @@ void freeKeyMaterial(key_material_t *keyMaterial) {
     if(keyMaterial->bigQ != NULL) {
       free(keyMaterial->bigQ);
     }
+    if(keyMaterial->bigDMP1 != NULL) {
+      free(keyMaterial->bigDMP1);
+    }
+    if(keyMaterial->bigDMQ1 != NULL) {
+      free(keyMaterial->bigDMQ1);
+    }
+    if(keyMaterial->bigIQMP != NULL) {
+      free(keyMaterial->bigIQMP);
+    }
+
     free(keyMaterial);
   }
 }
@@ -794,19 +1047,19 @@ CK_OBJECT_HANDLE searchObject(CK_SESSION_HANDLE hSession, char *objID, int objID
     { CKA_ID,    objID,   objIDLen }
   };
 
-  CK_RV rv = C_FindObjectsInit(hSession, objTemplate, 2);
+  CK_RV rv = p11->C_FindObjectsInit(hSession, objTemplate, 2);
   if(rv != CKR_OK) {
     fprintf(stderr, "Error: Could not prepare the object search.\n");
     return CK_INVALID_HANDLE;
   }
 
-  rv = C_FindObjects(hSession, &hObject, 1, &objectCount);
+  rv = p11->C_FindObjects(hSession, &hObject, 1, &objectCount);
   if(rv != CKR_OK) {
     fprintf(stderr, "Error: Could not get the search results.\n");
     return CK_INVALID_HANDLE;
   }
 
-  rv = C_FindObjectsFinal(hSession);
+  rv = p11->C_FindObjectsFinal(hSession);
   if(rv != CKR_OK) {
     fprintf(stderr, "Error: Could not finalize the search.\n");
     return CK_INVALID_HANDLE;
