@@ -1,4 +1,4 @@
-/* $Id: softhsm.h 2693 2010-01-22 09:17:08Z rb $ */
+/* $Id: softhsm.h 3905 2010-09-09 11:03:16Z rb $ */
 
 /*
  * Copyright (c) 2008-2009 .SE (The Internet Infrastructure Foundation).
@@ -33,12 +33,8 @@
 #include <sqlite3.h>
 
 // Includes for the crypto library
-#include <botan/auto_rng.h>
 #include <botan/pk_keys.h>
-#include <botan/rsa.h>
-#include <botan/pkcs8.h>
 #include <botan/bigint.h>
-#include <botan/if_algo.h>
 using namespace Botan;
 
 typedef struct key_material_t {
@@ -47,22 +43,34 @@ typedef struct key_material_t {
   CK_ULONG sizeD;
   CK_ULONG sizeP;
   CK_ULONG sizeQ;
+  CK_ULONG sizeDMP1;
+  CK_ULONG sizeDMQ1;
+  CK_ULONG sizeIQMP;
   CK_VOID_PTR bigE;
   CK_VOID_PTR bigN;
   CK_VOID_PTR bigD;
   CK_VOID_PTR bigP;
   CK_VOID_PTR bigQ;
+  CK_VOID_PTR bigDMP1;
+  CK_VOID_PTR bigDMQ1;
+  CK_VOID_PTR bigIQMP;
   key_material_t() {
     sizeE = 0;
     sizeN = 0;
     sizeD = 0;
     sizeP = 0;
     sizeQ = 0;
+    sizeDMP1 = 0;
+    sizeDMQ1 = 0;
+    sizeIQMP = 0;
     bigE = NULL_PTR;
     bigN = NULL_PTR;
     bigD = NULL_PTR;
     bigP = NULL_PTR;
     bigQ = NULL_PTR;
+    bigDMP1 = NULL_PTR;
+    bigDMQ1 = NULL_PTR;
+    bigIQMP = NULL_PTR;
   }
 } key_material_t;
 
@@ -73,6 +81,7 @@ void initToken(char *slot, char *label, char *soPIN, char *userPIN);
 void showSlots();
 void importKeyPair(char *filePath, char *filePIN, char *slot, char *userPIN, char *objectLabel, char *objectID, int forceExec);
 void exportKeyPair(char *filePath, char *filePIN, char *slot, char *userPIN, char *objectID);
+void optimize(char *slot, char *userPIN);
 
 // Support functions
 
@@ -89,9 +98,13 @@ Private_Key* getPrivKey(char *dbPath, CK_OBJECT_HANDLE oHandle);
 CK_KEY_TYPE getKeyType(sqlite3_stmt *select_an_attribute_sql, CK_OBJECT_HANDLE objectRef);
 CK_OBJECT_CLASS getObjectClass(sqlite3_stmt *select_an_attribute_sql, CK_OBJECT_HANDLE objectRef);
 BigInt getBigIntAttribute(sqlite3_stmt *select_an_attribute_sql, CK_OBJECT_HANDLE objectRef, CK_ATTRIBUTE_TYPE type);
+int removeSessionObjs(char *dbPath);
 
 /// Config
 char* getDBPath(CK_SLOT_ID slotID);
+CK_C_GetFunctionList loadLibrary(char *module);
+static void *moduleHandle;
+static CK_FUNCTION_LIST_PTR p11;
 
 /// PKCS#11 support
 CK_OBJECT_HANDLE searchObject(CK_SESSION_HANDLE hSession, char *objID, int objIDLen);
