@@ -1,4 +1,4 @@
-/* $Id: SoftSession.h 3904 2010-09-09 09:07:59Z rb $ */
+/* $Id: SoftSession.h 4671 2011-03-31 08:59:37Z rb $ */
 
 /*
  * Copyright (c) 2008-2009 .SE (The Internet Infrastructure Foundation).
@@ -82,10 +82,6 @@ class SoftSession {
     bool signSinglePart;
     CK_ULONG signSize;
     bool signInitialized;
-#ifdef SOFTHSM_SIGVER
-    PK_Verifier *pkSigVer;
-    MemoryRegion<byte> *pkSigVerData;
-#endif
 
     // Verify
     PK_Verifier *pkVerifier;
